unit LPTUNIT;

{ Unit mit Sammlung von Prozeduren und Funktionen zur Ansteuerung der Daten-,
  Steuer- und Statusleitungen des LPT-Interfaces und der angeschlossenen
  Peripherie ber die parallele Schnittstelle

  CopyRight Helmut Berka, DL2MAJ, Beethovenstrae 27a, 86836 Klosterlechfeld
  All Rights Reserved - Nichtkommerzielle Nutzung in Schule, Ausbildung
  und im Amateurfunk gestattet }

{ ------------------------------------------------------------------------- }
{ ------------------------------------------------------------------------- }

interface
{ ------------------------------------------------------------------------- }

uses DOS, CRT, PRINTER, SUPPORT, CURSOR;

type Pegel       = (lo, HI, Er);
     PegelLage   = (INV_, NINV);
     SchreibLies = (Schreiben, Lesen);

var LPT, j                          : integer;
    DataOut, ChipSelect,
    AdrOut, p2                      : byte;
    LPTadr                          : array[1 .. 4] of word;
    DatRegAdr, CtrlRegAdr,
    StatRegAdr                      : word;
    DatReg, CtrlReg, StatReg        : byte;
    Sel                             : array[0 .. 15] of byte;

{ ------------------------------------------------------------------------- }

procedure LPTs;
{ Routine stellt vorhandene LPT-Schnittstellen fest }
var LPTZahl                     : byte;
    LPTZahlStr, istr, W         : string;
    LPTadrStr                   : array[1 .. 4] of string;
    i, SteuerZ, x1              : integer;
{ ------------------------------------------------------------------------- }



function BITPEGEL(Data:byte; Lage: PegelLage; BitNr: integer) : Pegel;
{ Data : 8bit-Date, aus der ein Bit auszugeben ist
  Lage : INV_ oder NINV, je nachdem ob Ausgangssignal zu Registerinhalt
         invertiert oder nichtinvertiert ist
  BitNr: 0 .. 7, entsprechend z.B. Datenbit D0 .. D7                      }
{ ------------------------------------------------------------------------- }

function STROBEPEGEL : Pegel;
{ STROBE ist Bit 0, Wertigkeit = 2hoch0 = 1, Bit ist invertiert }
{ ------------------------------------------------------------------------- }

function AUTOFEEDPEGEL : Pegel;
{ AUTOFEED ist Bit 1, Wertigkeit = 2hoch1 = 2, Bit ist invertiert }
{ ------------------------------------------------------------------------- }

function INITPEGEL : Pegel;
{ INIT ist Bit 2, Wertigkeit = 2hoch2 = 4 }
{ ------------------------------------------------------------------------- }

function SLCTINPEGEL : Pegel;
{ SLCTIN ist Bit 3, Wertigkeit = 2hoch3 = 8 }
{ ------------------------------------------------------------------------- }

function IRQENABLEPEGEL : Pegel;
{ IRQENABLE ist Bit 4, Wertigkeit = 2hoch4 = 16
  es gibt allerdings keine Leitung mit diesem Namen !!!!! }
{ ------------------------------------------------------------------------- }


function ERRORPEGEL : Pegel;
{ ERROR ist Bit 3, Wertigkeit = 2hoch3 = 8 }
{ ------------------------------------------------------------------------- }

function SELECTPEGEL : Pegel;
{ SELECT ist Bit 4, Wertigkeit = 2hoch4 = 16 }
{ ------------------------------------------------------------------------- }

function PEPEGEL : Pegel;
{ PE ist Bit 5, Wertigkeit = 2hoch5 = 32 }
{ ------------------------------------------------------------------------- }

function ACKPEGEL : Pegel;
{ ACK ist Bit 6, Wertigkeit = 2hoch6 = 64 }
{ ------------------------------------------------------------------------- }

function BUSYPEGEL : Pegel;
{ BUSY ist Bit 7, Wertigkeit = 2hoch7 = 128, Bit ist 2x invertiert }
{ ------------------------------------------------------------------------- }



procedure STROBE(Zustand : Pegel);
{ STROBE ist Bit 0, Wertigkeit = 2hoch0 = 1, Bit ist invertiert }
{ ------------------------------------------------------------------------- }

procedure AUTOFEED(Zustand : Pegel);
{ AUTOFEED ist Bit 1, Wertigkeit = 2hoch1 = 2, Bit ist invertiert }
{ ------------------------------------------------------------------------- }

procedure INIT(Zustand : Pegel);
{ INIT ist Bit 2, Wertigkeit = 2hoch2 = 4 }
{ ------------------------------------------------------------------------- }

procedure SLCTIN(Zustand : Pegel);
{ SLCTIN ist Bit 3, Wertigkeit = 2hoch3 = 8 }
{ ------------------------------------------------------------------------- }

procedure IRQENABLE(Zustand : Pegel);
{ IRQENABLE ist Bit 4, Wertigkeit = 2hoch4 = 16 }
{ ------------------------------------------------------------------------- }

procedure AUSWAHL(SelxxNr, Kanal : integer);
{SelxxNr enthaelt Nr der gewuenschten Selxx-Leitung
 Kanal enthaelt Nr des gewuenschten Kanals          }
{ ------------------------------------------------------------------------- }

procedure DISPZUSTAND(NeuX, NeuY    : integer;
                      NeuAttr       : str4;
                      Leitung       : Pegel);
{ NeuX             = X-Koordinate der Bildschirmposition fr Anzeige
  NeuY             = Y-Koordinate der Bildschirmposition fr Anzeige
  Leitung          = Leitung, deren Zustand angezeigt wird                  }
{ ------------------------------------------------------------------------- }

function LPTin(SelxxNr, Kanal : integer) : byte;
{ Liest ein Byte ueber die LPT-Schnittstelle ein }
{ ------------------------------------------------------------------------- }

procedure LPTout(SelxxNr, Kanal, Wert : integer);
{ Gibt ein Byte ueber die LPT-Schnittstelle aus }
{ ------------------------------------------------------------------------- }

procedure LPTinout(SelxxNr, Kanal : integer; Aktion : SchreibLies; Wert : integer);
{ Gibt ein Byte ueber die LPT-Schnittstelle aus oder liest ein Byte ein}
{ ------------------------------------------------------------------------- }

procedure TEST;
{ ------------------------------------------------------------------------- }
{ ------------------------------------------------------------------------- }


{ ------------------------------------------------------------------------- }

implementation
{ ------------------------------------------------------------------------- }

procedure LPTs;
{ Routine stellt vorhandene LPT-Schnittstellen fest }
var LPTZahl                     : byte;
    LPTZahlStr, istr, W         : string;
    LPTadrStr                   : array[1 .. 4] of string;
    i, SteuerZ, x1              : integer;

begin
     LPTZahl := mem[$40:$11] shr 6;
     if LPTZahl = 0 then
        begin
           DISP(10,5,INV,'Keine parallele Schnittstelle vorhanden',0);
           halt;
        end
     else
        begin
           if LPTZahl = 1 then
           begin
              DISP(5,5,INV,'Nur 1 parallele Schnittstelle vorhanden !',0);
              DISP(5,7,INV,'Bitte einen an LPT1 angeschlossenen Drucker abstecken,',0);
              DISP(5,8,INV,'zum Schutz vor evtl. mglichen Beschdigung des Druckers,',0);
              DISP(5,9,INV,'da die parallele Schnittstelle nicht als Druckerschnittstelle',0);
              DISP(5,10,INV,'angesteuert wird (auch nicht bers BIOS).',0);
              DISP(5,12,HIGH,'Zum Fortfahren (=kein Drucker an LPT1) bitte  J  und <RETURN>',0);
              DISP(5,13,HIGH,'Zum Programmabbruch bitte <RETURN> eingeben',0);
              W := 'N';
              LIESLN(7,40,1,INV,W,SteuerZ);
              if W <> upcase('j') then halt;
           end;

           for i := 1 to LPTZahl do LPTadr[i]  := memw[$40:$06+(2*i)];

           str(LPTZahl,LPTZahlStr);
           DISP(3,2,HIGH,'Anzahl der parallelen Schnittstellen : ',0);
           DISP(42,2,HIGH,LPTZahlStr,0);

           for i := 1 to LPTZahl do

           begin
              str(i,istr);
              DISP(10,3+i,HIGH,'Adresse von LPT' + istr + ' : ',0);
              DISP(52,3+i,HIGH,WordToHex(LPTadr[i]) + 'H',0);
           end;

           DISP(10,5+LPTZahl,HIGH,'Bitte whlen Sie : ',0);
           LPT := LPTZahl;
           LIESINT(32,5+LPTZahl,2,High,LPT,SteuerZ);

        end;
        clrscr;
end;
{ ------------------------------------------------------------------------- }



function BITPEGEL(Data:byte; Lage: PegelLage; BitNr: integer) : Pegel;
{ Data : 8bit-Date, aus der ein Bit auszugeben ist
  Lage : INV_ oder NINV, je nachdem ob Ausgangssignal zu Registerinhalt
         invertiert oder nichtinvertiert ist
  BitNr: 0 .. 7, entsprechend z.B. Datenbit D0 .. D7                      }

begin
   BitNr := 8 - BitNr; { Bit0 ist LSB und steht ganz rechts = 8. Stelle }
   if ((BitNr < 1) or (BitNr > 8))
      then BITPEGEL := Er
      else if Lage = INV_
              then if copy(DEZBIN(Data),BitNr,1) = '0'
                      then BITPEGEL := HI
                      else BITPEGEL := lo
              else if copy(DEZBIN(Data),BitNr,1) = '0'
                      then BITPEGEL := lo
                      else BITPEGEL := HI;
end;
{ ------------------------------------------------------------------------- }


function STROBEPEGEL : Pegel;

begin
   STROBEPEGEL := BITPEGEL(CtrlReg,INV_,0);
end;
{ ------------------------------------------------------------------------- }


function AUTOFEEDPEGEL : Pegel;

begin
   AUTOFEEDPEGEL := BITPEGEL(CtrlReg,INV_,1);
end;
{ ------------------------------------------------------------------------- }


function INITPEGEL : Pegel;

begin
   INITPEGEL := BITPEGEL(CtrlReg,NINV,2);
end;
{ ------------------------------------------------------------------------- }


function SLCTINPEGEL : Pegel;

begin
   SLCTINPEGEL := BITPEGEL(CtrlReg,INV_,3);
end;
{ ------------------------------------------------------------------------- }


function IRQENABLEPEGEL : Pegel;

begin
   IRQENABLEPEGEL := BITPEGEL(CtrlReg,NINV,4);
end;
{ ------------------------------------------------------------------------- }


function ERRORPEGEL : Pegel;

begin
   ERRORPEGEL := BITPEGEL(StatReg,NINV,3);
end;
{ ------------------------------------------------------------------------- }


function PEPEGEL : Pegel;

begin
   PEPEGEL := BITPEGEL(StatReg,NINV,4);
end;
{ ------------------------------------------------------------------------- }


function SELECTPEGEL : Pegel;

begin
   SELECTPEGEL := BITPEGEL(StatReg,NINV,5);
end;
{ ------------------------------------------------------------------------- }


function ACKPEGEL : Pegel;

begin
   ACKPEGEL := BITPEGEL(StatReg,NINV,6);
end;
{ ------------------------------------------------------------------------- }


function BUSYPEGEL : Pegel;

begin
   BUSYPEGEL := BITPEGEL(StatReg,NINV,7);
end;
{ ------------------------------------------------------------------------- }




procedure STROBE(Zustand : Pegel);
{ STROBE ist Bit 0, Wertigkeit = 2hoch0 = 1, Bit ist invertiert }
begin
   if Zustand = lo then CtrlReg := CtrlReg or 1    { Bit ist invertiert }
                   else CtrlReg := CtrlReg and 254;
   port[CtrlRegAdr] := CtrlReg;
end;
{ ------------------------------------------------------------------------- }


procedure AUTOFEED(Zustand : Pegel);
{ AUTOFEED ist Bit 1, Wertigkeit = 2hoch1 = 2, Bit ist invertiert }
begin
   if Zustand = lo then CtrlReg := CtrlReg or 2    { Bit ist invertiert }
                   else CtrlReg := CtrlReg and 253;
   port[CtrlRegAdr] := CtrlReg;
end;
{ ------------------------------------------------------------------------- }


procedure INIT(Zustand : Pegel);
{ INIT ist Bit 2, Wertigkeit = 2hoch2 = 4 }
begin
   if Zustand = lo then CtrlReg := CtrlReg and 251 { Bit nicht invertiert }
                   else CtrlReg := CtrlReg or 4;
   port[CtrlRegAdr] := CtrlReg;
end;
{ ------------------------------------------------------------------------- }


procedure SLCTIN(Zustand : Pegel);
{ SLCTIN ist Bit 3, Wertigkeit = 2hoch3 = 8, Bit ist invertiert }
begin
   if Zustand = lo then CtrlReg := CtrlReg or 8    { Bit ist invertiert }
                   else CtrlReg := CtrlReg and 247;
   port[CtrlRegAdr] := CtrlReg;
end;
{ ------------------------------------------------------------------------- }


procedure IRQENABLE(Zustand : Pegel);
{ IRQENABLE ist Bit 4, Wertigkeit = 2hoch4 = 16 }
begin
   if Zustand = lo then CtrlReg := CtrlReg and 239 { Bit nicht invertiert }
                   else CtrlReg := CtrlReg or 16;
   port[CtrlRegAdr] := CtrlReg;
end;
{ ------------------------------------------------------------------------- }


procedure AUSWAHL(SelxxNr, Kanal : integer);
{SelxxNr enthaelt Nr der gewuenschten Selxx-Leitung
 Kanal enthaelt Nr des gewuenschten Kanals          }
begin
   AUTOFEED(HI);
   if ((SelxxNr<0) or (SelxxNr>15)) then SelxxNr := 15;
   if ((Kanal<0) or (Kanal>15)) then Kanal := 15;
   port[DatRegAdr] := Sel[SelxxNr] + Kanal;
   INIT(lo);
   INIT(HI);
   INIT(LO);
end;
{ ------------------------------------------------------------------------- }


procedure DISPZUSTAND(NeuX, NeuY    : integer;
                      NeuAttr       : str4;
                      Leitung       : Pegel);
{ NeuX             = X-Koordinate der Bildschirmposition fr Anzeige
  NeuY             = Y-Koordinate der Bildschirmposition fr Anzeige
  Leitung          = Leitung, deren Zustand angezeigt wird                  }
begin
   if Leitung = Er then DISP(NeuX, NeuY, NeuAttr, 'ER',2);
   if Leitung = lo then DISP(NeuX, NeuY, NeuAttr, 'lo',2)
                   else DISP(NeuX, NeuY, NeuAttr, 'HI',2);
end;
{ ------------------------------------------------------------------------- }

function LPTin(SelxxNr, Kanal : integer) : byte;
{ Liest ein Byte ueber die LPT-Schnittstelle ein }
var DataIn : byte;

begin
   AUSWAHL(SelxxNr, Kanal);
   STROBE(lo);
   SLCTIN(lo);
   DataIn := (port[StatRegAdr] shr 4) and $0F;
   SLCTIN(HI);
   LPTin := DataIn + (port[StatRegAdr] and $F0);
   STROBE(HI);
end;
{ ------------------------------------------------------------------------- }

procedure LPTout(SelxxNr, Kanal, Wert : integer);
{ Gibt ein Byte ueber die LPT-Schnittstelle aus }
begin
   AUSWAHL(SelxxNr, Kanal);
   port[DatRegAdr] := Wert;
   AUTOFEED(lo);
   STROBE(lo);
   STROBE(HI);
end;
{ ------------------------------------------------------------------------- }

procedure LPTinout(SelxxNr, Kanal : integer; Aktion : SchreibLies; Wert : integer);
{ Gibt ein Byte ueber die LPT-Schnittstelle aus oder liest ein Byte ein}
begin
   if Aktion = Schreiben then LPTout(SelxxNr, Kanal+8, Wert)
                         else Wert := LPTin(SelxxNr, Kanal)
end;
{ ------------------------------------------------------------------------- }


procedure TEST;

var ch : char;

begin
   DISP(5,22,LOW,' ',74);
   DISP(23,22,HIGH,'Mit beliebiger Taste Test beenden!',0);
   AUTOFEED(lo);

   repeat
      DatReg := 1;
      port[DatRegAdr] := DatReg;
      delay(499);
      DatReg := 0;
      port[DatRegAdr] := DatReg;
      delay(499);
   until keypressed;

   ch := readkey;

end;
{ ------------------------------------------------------------------------- }




{ ------------------------------------------------------------------------- }
{ *********************** PROGRAMMTEIL ************************************ }
{ ------------------------------------------------------------------------- }
begin
   clrscr;

   { voreingestellte Werte bei Programmstart             }


   Sel[ 0] :=   0;
   Sel[ 1] := 128;
   Sel[ 2] :=  64;
   Sel[ 3] := 192;
   Sel[ 4] :=  32;
   Sel[ 5] := 160;
   Sel[ 6] :=  96;
   Sel[ 7] := 224;
   Sel[ 8] :=  16;
   Sel[ 9] := 144;
   Sel[10] :=  80;
   Sel[11] := 208;
   Sel[12] :=  48;
   Sel[13] := 176;
   Sel[14] := 112;
   Sel[15] := 240;

   LPT           := 2;     { Nr. der parallelen Schnittstellen }
   DatReg        := $00;
   AdrOut        := $FF;
   DataOut       := $87;
   CtrlReg       := $FF;

   LPTs;

   DatRegAdr     := LPTadr[LPT];
   StatRegAdr    := DatRegAdr + 1;
   CtrlRegAdr    := DatRegAdr + 2;

   port[CtrlRegAdr] := CtrlReg;
   port[DatRegAdr]  := DatReg;
end.
{ -- Ende der UNIT LPTUNIT.PAS -------------------------------------------- }
