unit SUPPORT;
{ ------------------------------------------------------------------------- }
{ ------------------------------------------------------------------------- }

interface
{ ------------------------------------------------------------------------- }

uses CRT;

const  INV  = 'INV ';
       HIGH = 'HIGH';
       LOW  = 'LOW ';

type str1 = string[1];
     str2 = string[2];
     str4 = string[4];
     str8 = string[8];

{ ------------------------------------------------------------------------- }

procedure BEEP(Frequenz, Dauer : integer);

{ Frequenz         = Frequenz des erzeugten Tons in Hz
  Dauer            = Dauer des Tons in msec                                 }
{ ------------------------------------------------------------------------- }

function DEZBIN(ByteInhalt : Byte) : str8;

{ ------------------------------------------------------------------------- }

function ByteToHex (h : Byte) : str2;      { converts a Byte to 2 Hex Chars }

{ ------------------------------------------------------------------------- }

procedure DISP(NeuX, NeuY    : integer;
               NeuAttr       : str4;
               Ausdruck      : string;
               StellenGesamt : integer);

{ NeuX             = X-Koordinate der Bildschirmposition fr Anzeige
  NeuY             = Y-Koordinate der Bildschirmposition fr Anzeige
  Ausdruck         = Zeichenfolge, die angezeigt wird
  StellenGesamt    = Anzahl der anzuzeigende Stellen,
                     bei 0 wird Stringlnge berechnet                       }
{ ------------------------------------------------------------------------- }

procedure DISPR(NeuX, NeuY                      : integer;
                NeuAttr                         : str4;
                RealZahl                        : real;
                StellenGesamt, StellenNachkomma : integer);

{ NeuX             = X-Koordinate der Bildschirmposition fr Anzeige
  NeuY             = Y-Koordinate der Bildschirmposition fr Anzeige
  RealZahl         = Zahl, die angezeigt wird
  StellenGesamt    = Vorzeichen + Integeranteil + Komma + Nachkommaanteil
  StellenNachKomma = Anzahl der anzuzeigende Stellen nach dem Komma        }
{ ------------------------------------------------------------------------- }

procedure LIESLN (    x, y, Laenge : integer;
                      Attr         : str4;
                  var Wort         : string;
                  var Steuerchar   : integer);
{ ------------------------------------------------------------------------- }

procedure LIESLONGINT (     x2, y2, StellenGesamt : integer;
                            Attr                  : str4;
                            var Zahl              : longint;
                            var Steuerzeichen     : integer);

{ x2               = X-Koordinate der Bildschirmposition fr Anzeige
  y2               = Y-Koordinate der Bildschirmposition fr Anzeige
  StellenGesamt    = Vorzeichen + Integeranteil
  Zahl             = Zahl, die eingegeben werden soll
  Steuerzeichen    = Variable fr zuletzt gedrckte Funktionstaste         }

{ ------------------------------------------------------------------------- }

procedure LIESREAL (    x2, y2, StellenGesamt, StellenNachKomma : integer;
                        Attr                                    : str4;
                    var RealZahl                                : real;
                    var Steuerzeichen                           : integer);
{ ------------------------------------------------------------------------- }

procedure LIESINT (     x2, y2, StellenGesamt : integer;
                        Attr                  : str4;
                    var Zahl                  : integer;
                    var Steuerzeichen         : integer);

{ x2               = X-Koordinate der Bildschirmposition fr Anzeige
  y2               = Y-Koordinate der Bildschirmposition fr Anzeige
  StellenGesamt    = Vorzeichen + Integeranteil
  Zahl             = Zahl, die eingegeben werden soll
  Steuerzeichen    = Variable fr zuletzt gedrckte Funktionstaste         }

{ ------------------------------------------------------------------------- }

procedure RAHMEN(x1,y1,x2,y2 : integer;
                 Attr        : str4;
                 Doppelt     : boolean    );

{ x1, y1  : linke, obere Ecke des Rahmens    0 < x < 81,  0 < y < 25
  x2, y2  : rechte, untere Ecke des Rahmens  0 < x < 81,  0 < y < 25
  Attr    : Bildschirmattribut
  Doppelt : Rahmen aus doppelter (=true) oder einfacher Linie zeichnen      }

{ ------------------------------------------------------------------------- }

procedure WEITER;

{ entspricht MS-DOS-Befehl PAUSE                                            }
{ ------------------------------------------------------------------------- }

function WordToHex (w : Word) : str4;      { converts a Word to 4 Hex Chars }

{ ------------------------------------------------------------------------- }


function ZahlToStr(B: longint;L: integer): String;
{ Convert any integer type to a string
  B : Zahl, die in einen string umzuwandeln ist
  L : Lnge des strings                          }
{ ------------------------------------------------------------------------- }
{ ------------------------------------------------------------------------- }


implementation
{ ------------------------------------------------------------------------- }

procedure BEEP(Frequenz, Dauer : integer);

begin
     sound(Frequenz);
     delay(Dauer);
     nosound
end;
{ ------------------------------------------------------------------------- }


function DEZBIN(ByteInhalt : Byte) : str8;

var Rest     : integer;
    Restchar : str1;
    BinZahl  : str8;


begin
  BinZahl := '';

  repeat
     Rest       := ByteInhalt mod 2;
     ByteInhalt := ByteInhalt div 2;
     str(Rest,Restchar);
     BinZahl := Restchar + BinZahl;
  until ByteInhalt = 0;

  while length(BinZahl) <> 8 do BinZahl := '0' + BinZahl;
  DEZBIN := BinZahl
end;
{ ------------------------------------------------------------------------- }


function ByteToHex (h : Byte) : str2;      { converts a Byte to 2 Hex Chars }

const HexArray : array[0..15] of char = '0123456789ABCDEF';

begin
   ByteToHex := HexArray[h shr 4] + HexArray[h and $F]
end;
{ ------------------------------------------------------------------------- }


procedure DISP(NeuX, NeuY    : integer;
               NeuAttr       : str4;
               Ausdruck      : string;
               StellenGesamt : integer);

var AltAttr   : byte;
    NeuAttrok : boolean;
    i         : integer;

begin
   AltAttr := CRT.TextAttr;
   if StellenGesamt = 0 then StellenGesamt := length(Ausdruck)
                        else while length(Ausdruck) < StellenGesamt
                             do Ausdruck := Ausdruck + #32;
   NeuAttrok := false;
   if NeuAttr = 'LOW ' then begin LowVideo;  NeuAttrok := true end;
   if NeuAttr = 'HIGH' then begin HighVideo; NeuAttrok := true end;
   if NeuAttr = 'INV ' then begin
                               TextBackground(LightGray);
                               TextColor(Black);
                               NeuAttrok := true
                            end;

   if NeuAttrok = false then writeln(#7);

   gotoxy(NeuX,NeuY);
   write(Ausdruck);
   gotoxy(NeuX,NeuY);
   CRT.TextAttr := AltAttr
end;
{ ------------------------------------------------------------------------- }


procedure DISPR(NeuX, NeuY                      : integer;
                NeuAttr                         : str4;
                RealZahl                        : real;
                StellenGesamt, StellenNachkomma : integer);

{ NeuX             = X-Koordinate der Bildschirmposition fr Anzeige
  NeuY             = Y-Koordinate der Bildschirmposition fr Anzeige
  RealZahl         = Zahl, die angezeigt wird
  StellenGesamt    = Vorzeichen + Integeranteil + Komma + Nachkommaanteil
  StellenNachKomma = Anzahl der anzuzeigende Stellen nach dem Komma        }


var
    Vorzeichen  : char;       { Vorzeichen der anzuzeigenden Realzahl      }
    i           : integer;    { Hilfsvariable                              }
    MaxZahl     : real;       { max. darstellbare Zahl                     }
    Max         : string;     { Ausdruck fr berlaufanzeige               }
    AltAttr     : byte;       { Bildschirmattribut vor und nach Anzeige    }
    NeuAttrok   : boolean;    { Bildschirmattribut fr Anzeige erkannt     }


begin
   { Kontolle und evtl. Klingeln und "Korrektur", wenn Fehler bei Stellen-
     angaben in bergebenen Parametern                                      }
   if StellenGesamt < (StellenNachKomma + 2) then
                 begin StellenGesamt := StellenNachKomma + 2; write(#7) end;

   if StellenGesamt < 2 then begin StellenGesamt := 2; write(#7) end;

   { max. darstellbare Zahl entsprechend StellenGesamt und StellenNachKomma
     feststellen                                                            }
   Max := '';
   if StellenNachKomma = 0

      then  { hier : nur Integeranteil }
         while length(Max) < (StellenGesamt - 1) do Max := Max + '9'

      else  { hier : echte Realzahl }

         begin
            while length(Max) < (StellenGesamt - StellenNachkomma - 2) do
                            Max := Max + '9';  { Integeranteil erzeugen }

            Max := Max + '.';                  { Integeranteil + Komma  }
            while length(Max) < (StellenGesamt - 1) do Max := Max + '9';
                              { Integeranteil + Komma + Nachkommaanteil }
         end;

   val(Max,MaxZahl,i);        { Max=string in MaxZahl=Realzahl umwandeln }

   { Max fr berlaufanzeige aufbereiten }
   if RealZahl >= 0 then Max := #32 + Max else Max := '-' + Max;
   if RealZahl >= 0 then Max := '>' + Max else Max := '<' + Max;

   { Max durch '>' bzw. '<' zu lang, daher krzen }
   Max := copy(Max,1,length(Max)-1);

   AltAttr := CRT.TextAttr;  { bestehendes Bildschirmattribut sichern }

   { Bildschirmattribut fr Ausgabe setzen }
   NeuAttrok := false;
   if NeuAttr = 'LOW ' then begin LowVideo;  NeuAttrok := true end;
   if NeuAttr = 'HIGH' then begin HighVideo; NeuAttrok := true end;
   if NeuAttr = 'INV ' then begin
                               TextBackground(LightGray);
                               TextColor(Black);
                               NeuAttrok := true
                            end;

   { Klingeln, wenn Attribut nicht erkannt! }
   if NeuAttrok = false then writeln(#7);

   gotoxy(NeuX,NeuY);  { Cursor auf Ausgabeposition setzen }

   if abs(RealZahl) <= MaxZahl then

   begin  { RealZahl ist darstellbar }
      if RealZahl < 0 then Vorzeichen := '-' else Vorzeichen := #32;
      writeln(abs(RealZahl):StellenGesamt:StellenNachKomma);
      gotoxy(NeuX,NeuY);
      write(Vorzeichen)
   end

   else write(Max);  { RealZahl entweder zu gro oder zu negativ! }
   CRT.TextAttr := AltAttr  { altes Bildschirmattribut wieder einstellen }
end;
{ ------------------------------------------------------------------------- }


procedure LIESLN (    x, y, Laenge : integer;
                      Attr         : str4;
                  var Wort         : string;
                  var Steuerchar   : integer);

var OrigWort, W1, Leer : string;
    l            : integer;
    ch, ch1      : char;

begin
   OrigWort := Wort;
   Steuerchar := 0;
   Leer := '';
   for l:=1 to Laenge do Leer := Leer + #32;
   DISP(x,y,Attr,Leer,0);
   W1 := '';
   Wort:=copy(Wort,1,Laenge);
   DISP(x,y,Attr,Wort,0);
   gotoxy(x,y);

   repeat
      ch := readkey;

      case ord(ch) of

      { *********** Cursor-, Funktionstasten, Shift-Tab, CR, ESC *********** }

         0, 9, 13, 27 : begin
                           Steuerchar := 0;
                           if ch = #0 then

                           begin
                              ch := readkey;
                              Steuerchar := -ord(ch)
                           end

                           else  Steuerchar := +ord(ch)
                        end;


      { *********** Back Space, alle auf Bildschirm darstellbare Zeichen *** }

         8, 32..127   : begin
                           if ch = #8
                              then W1 := copy(W1,1,(length(W1)-1))
                              else if length(W1) < Laenge then W1 := W1 + ch
                                                          else write(#7);
                           DISP(x,y,Attr,Leer,0);
                           DISP(x,y,Attr,W1,0);
                           if length(W1) < Laenge
                              then gotoxy((x + length(W1)),y)
                              else gotoxy((x - 1 + length(W1)),y);
                           Wort := W1
                        end

      end;

   until (Steuerchar = 27) or (Steuerchar = 13) or (Steuerchar < 0);

   if Steuerchar = 27 then Wort := OrigWort

end;
{ ------------------------------------------------------------------------- }


procedure LIESLONGINT (     x2, y2, StellenGesamt : integer;
                            Attr                  : str4;
                            var Zahl              : longint;
                            var Steuerzeichen     : integer);

{ x2               = X-Koordinate der Bildschirmposition fr Anzeige
  y2               = Y-Koordinate der Bildschirmposition fr Anzeige
  StellenGesamt    = Vorzeichen + Integeranteil
  Zahl             = Zahl, die eingegeben werden soll
  Steuerzeichen    = Variable fr zuletzt gedrckte Funktionstaste         }

var  Fehler, i        : integer;    { Hilfsvariablen                        }
     w1, w2, Wort3    : string[15]; { Hilfsvariablen                        }
     OrigZahl         : integer;    { Wert von RealZahl vor Neueingabe      }
     Leer             : string;     { Leerstring mit Lnge StellenGesamt    }
     ch               : char;       { Zeichen, das eingelesen wird          }
     ZahlNeu          : boolean;    { Hilfsvar. fr bernahme von OrigZahl  }
     StellenNachKomma : integer;    { Hilfsvariable                         }

begin
   StellenNachKomma := 0;
   OrigZahl := Zahl;    { bergebene Zahl sichern }
   ZahlNeu := false;    { true, wenn Zahl neu eingegeben und mit CR beendet }
   DISPR(x2,y2,Attr,Zahl,StellenGesamt,StellenNachKomma);
   Leer := '';
   for i := 1 to StellenGesamt do Leer := Leer + #32;

   repeat
      Steuerzeichen := 0;
      gotoxy(x2,y2);
      w1 := '';
      Wort3 := '';

      repeat
         ch := readkey;

         case ord(ch) of

         { *********** Cursor-, Funktionstasten, Shift-Tab, CR, ESC ******** }

            0, 9, 13, 27 : begin
                              if ch = #0 then

                              begin
                                 ch := readkey;
                                 Steuerzeichen := -ord(ch)
                              end

                              else  Steuerzeichen := +ord(ch)
                           end;


         { *********** Back Space/0..9/e/E/+/-/,/. ************************* }

            8, 43..46,
            48..57, 69,
            101          : begin
                              if ch = #8  { Back Space }
                                 then w1 := copy(w1,1,(length(w1)-1))
                                 else if length(w1) < StellenGesamt
                                         then w1 := w1 + ch else write(#7);
                              DISP(x2,y2,Attr,Leer,0);
                              DISP(x2,y2,Attr,w1,0);
                              if length(w1) < StellenGesamt
                                 then gotoxy((x2 + length(w1)),y2)
                                 else gotoxy((x2 - 1 + length(w1)),y2);
                              Wort3 := w1;
                              ZahlNeu := true
                           end

            else write(#7)  { klingeln, wenn ungltiges Zeichen eingegeben }

         end;

      until (Steuerzeichen = 27) or (Steuerzeichen = 13) or
            (Steuerzeichen =  9) or (Steuerzeichen < 0);

      if (Steuerzeichen = 13) and (Wort3 <> '') then

      begin
         i := 1;
         w2 := '';

         repeat          { Falls KOMMA eingegeben, in PUNKT umwandeln }
            w1 := copy(Wort3,i,1);
            if w1 = ',' then w1 := '.';
            w2 := w2 + w1;
            i := i + 1
         until i > length(Wort3);

         Wort3 := w2;
         val(Wort3,Zahl,Fehler)
      end

      else Fehler:=0;

   until (Fehler=0);

   if (Steuerzeichen = 27) or (ZahlNeu = false) then Zahl := OrigZahl;
   DISPR(x2,y2,Attr,Zahl,StellenGesamt,StellenNachKomma)
end;
{ ------------------------------------------------------------------------- }


procedure LIESREAL (    x2, y2, StellenGesamt, StellenNachKomma : integer;
                        Attr                                    : str4;
                    var RealZahl                                : real;
                    var Steuerzeichen                           : integer);

{ x2               = X-Koordinate der Bildschirmposition fr Eingabe
  y2               = Y-Koordinate der Bildschirmposition fr Eingabe
  Attr             = Attribut fr Bildschirmanzeige
  StellenGesamt    = Vorzeichen + Integeranteil + Komma + Nachkommaanteil
  StellenNachKomma = Anzahl der anzuzeigende Stellen nach dem Komma
  RealZahl         = Zahl, die eingegeben werden soll
  Steuerzeichen    = Variable fr zuletzt gedrckte Funktionstaste         }


var  Fehler, i     : integer;      { Hilfsvariablen                        }
     w1, w2, Wort3 : string[15];   { Hilfsvariablen                        }
     OrigZahl      : real;         { Wert von RealZahl vor Neueingabe      }
     Leer          : string;       { Leerstring mit Lnge StellenGesamt    }
     ch            : char;         { Zeichen, das eingelesen wird          }
     ZahlNeu       : boolean;      { Hilfsvar. fr bernahme von OrigZahl  }

begin
   OrigZahl := RealZahl;    { bergebene Zahl sichern }
   ZahlNeu := false;    { true, wenn Zahl neu eingegeben und mit CR beendet }
   DISPR(x2,y2,Attr,RealZahl,StellenGesamt,StellenNachKomma);
   Leer := '';
   for i := 1 to StellenGesamt do Leer := Leer + #32;

   repeat
      Steuerzeichen := 0;
      gotoxy(x2,y2);
      w1 := '';
      Wort3 := '';

      repeat
         ch := readkey;

         case ord(ch) of

         { *********** Cursor-, Funktionstasten, Shift-Tab, CR, ESC ******** }

            0, 9, 13, 27 : begin
                              if ch = #0 then

                              begin
                                 ch := readkey;
                                 Steuerzeichen := -ord(ch)
                              end

                              else  Steuerzeichen := +ord(ch)
                           end;


         { *********** Back Space/0..9/e/E/+/-/,/. ************************* }

            8, 43..46,
            48..57, 69,
            101          : begin
                              if ch = #8  { Back Space }
                                 then w1 := copy(w1,1,(length(w1)-1))
                                 else if length(w1) < StellenGesamt
                                         then w1 := w1 + ch else write(#7);
                              DISP(x2,y2,Attr,Leer,0);
                              DISP(x2,y2,Attr,w1,0);
                              if length(w1) < StellenGesamt
                                 then gotoxy((x2 + length(w1)),y2)
                                 else gotoxy((x2 - 1 + length(w1)),y2);
                              Wort3 := w1;
                              ZahlNeu := true
                           end

            else write(#7)  { klingeln, wenn ungltiges Zeichen eingegeben }

         end;

      until (Steuerzeichen = 27) or (Steuerzeichen = 13) or
            (Steuerzeichen =  9) or (Steuerzeichen < 0);

      if (Steuerzeichen = 13) and (Wort3 <> '') then

      begin
         i := 1;
         w2 := '';

         repeat          { Falls KOMMA eingegeben, in PUNKT umwandeln }
            w1 := copy(Wort3,i,1);
            if w1 = ',' then w1 := '.';
            w2 := w2 + w1;
            i := i + 1
         until i > length(Wort3);

         Wort3 := w2;
         val(Wort3,RealZahl,Fehler)
      end

      else Fehler:=0;

   until (Fehler=0);

   if (Steuerzeichen = 27) or (ZahlNeu = false) then RealZahl := OrigZahl;
   DISPR(x2,y2,Attr,RealZahl,StellenGesamt,StellenNachKomma)
end;
{ ------------------------------------------------------------------------- }


procedure LIESINT (     x2, y2, StellenGesamt : integer;
                        Attr                  : str4;
                    var Zahl                  : integer;
                    var Steuerzeichen         : integer);

{ x2               = X-Koordinate der Bildschirmposition fr Anzeige
  y2               = Y-Koordinate der Bildschirmposition fr Anzeige
  StellenGesamt    = Vorzeichen + Integeranteil
  Zahl             = Zahl, die eingegeben werden soll
  Steuerzeichen    = Variable fr zuletzt gedrckte Funktionstaste         }

var  Fehler, i        : integer;    { Hilfsvariablen                        }
     w1, w2, Wort3    : string[15]; { Hilfsvariablen                        }
     OrigZahl         : integer;    { Wert von RealZahl vor Neueingabe      }
     Leer             : string;     { Leerstring mit Lnge StellenGesamt    }
     ch               : char;       { Zeichen, das eingelesen wird          }
     ZahlNeu          : boolean;    { Hilfsvar. fr bernahme von OrigZahl  }
     StellenNachKomma : integer;    { Hilfsvariable                         }

begin
   StellenNachKomma := 0;
   OrigZahl := Zahl;    { bergebene Zahl sichern }
   ZahlNeu := false;    { true, wenn Zahl neu eingegeben und mit CR beendet }
   DISPR(x2,y2,Attr,Zahl,StellenGesamt,StellenNachKomma);
   Leer := '';
   for i := 1 to StellenGesamt do Leer := Leer + #32;

   repeat
      Steuerzeichen := 0;
      gotoxy(x2,y2);
      w1 := '';
      Wort3 := '';

      repeat
         ch := readkey;

         case ord(ch) of

         { *********** Cursor-, Funktionstasten, Shift-Tab, CR, ESC ******** }

            0, 9, 13, 27 : begin
                              if ch = #0 then

                              begin
                                 ch := readkey;
                                 Steuerzeichen := -ord(ch)
                              end

                              else  Steuerzeichen := +ord(ch)
                           end;


         { *********** Back Space/0..9/e/E/+/-/,/. ************************* }

            8, 43..46,
            48..57, 69,
            101          : begin
                              if ch = #8  { Back Space }
                                 then w1 := copy(w1,1,(length(w1)-1))
                                 else if length(w1) < StellenGesamt
                                         then w1 := w1 + ch else write(#7);
                              DISP(x2,y2,Attr,Leer,0);
                              DISP(x2,y2,Attr,w1,0);
                              if length(w1) < StellenGesamt
                                 then gotoxy((x2 + length(w1)),y2)
                                 else gotoxy((x2 - 1 + length(w1)),y2);
                              Wort3 := w1;
                              ZahlNeu := true
                           end

            else write(#7)  { klingeln, wenn ungltiges Zeichen eingegeben }

         end;

      until (Steuerzeichen = 27) or (Steuerzeichen = 13) or
            (Steuerzeichen =  9) or (Steuerzeichen < 0);

      if (Steuerzeichen = 13) and (Wort3 <> '') then

      begin
         i := 1;
         w2 := '';

         repeat          { Falls KOMMA eingegeben, in PUNKT umwandeln }
            w1 := copy(Wort3,i,1);
            if w1 = ',' then w1 := '.';
            w2 := w2 + w1;
            i := i + 1
         until i > length(Wort3);

         Wort3 := w2;
         val(Wort3,Zahl,Fehler)
      end

      else Fehler:=0;

   until (Fehler=0);

   if (Steuerzeichen = 27) or (ZahlNeu = false) then Zahl := OrigZahl;
   DISPR(x2,y2,Attr,Zahl,StellenGesamt,StellenNachKomma)
end;
{ ------------------------------------------------------------------------- }


procedure RAHMEN(x1,y1,x2,y2 : integer;
                 Attr        : str4;
                 Doppelt     : boolean    );

{ x1, y1  : linke, obere Ecke des Rahmens    0 < x < 81,  0 < y < 25
  x2, y2  : rechte, untere Ecke des Rahmens  0 < x < 81,  0 < y < 25
  Attr    : Bildschirmattribut
  Doppelt : Rahmen aus doppelter (=true) oder einfacher Linie zeichnen      }

var xy, Fehler, i : integer;

begin
   Fehler := 0;
   if x1 <  1 then begin x1 :=  1; Fehler :=  1; end;
   if x1 > 80 then begin x1 := 80; Fehler :=  2; end;
   if y1 <  1 then begin y1 :=  1; Fehler :=  3; end;
   if y1 > 24 then begin y1 := 24; Fehler :=  4; end;
   if x2 <  1 then begin x2 :=  1; Fehler :=  5; end;
   if x2 > 80 then begin x2 := 80; Fehler :=  6; end;
   if y2 <  1 then begin y2 :=  1; Fehler :=  7; end;
   if y2 > 80 then begin x2 := 80; Fehler :=  8; end;
   if x1 > x2 then begin xy := x1; x1 := x2; x2 := xy; Fehler :=  9; end;
   if y1 > y2 then begin xy := y1; y1 := y2; y2 := xy; Fehler := 10; end;
   if (Attr <> 'LOW ') and (Attr <> 'HIGH') and (Attr <> 'INV ')
      then begin Attr := 'LOW '; Fehler := 21; end;

   if Doppelt = true
      then begin
              DISP(x1,y1,Attr,'',1);
              DISP(x2,y1,Attr,'',1);

              for i := (x1+1) to (x2-1) do
              begin
                 DISP(i,y1,Attr,'',1);
                 DISP(i,y2,Attr,'',1);
              end;

              for i := (y1+1) to (y2-1) do
              begin
                 DISP(x1,i,Attr,'',1);
                 DISP(x2,i,Attr,'',1)
              end;

              DISP(x1,y2,Attr,'',1);
              DISP(x2,y2,Attr,'',1);
          end
      else begin
              DISP(x1,y1,Attr,'',1);
              DISP(x2,y1,Attr,'',1);

              for i := (x1+1) to (x2-1) do
              begin
                 DISP(i,y1,Attr,'',1);
                 DISP(i,y2,Attr,'',1);
              end;

              for i := (y1+1) to (y2-1) do
              begin
                 DISP(x1,i,Attr,'',1);
                 DISP(x2,i,Attr,'',1)
              end;

              DISP(x1,y2,Attr,'',1);
              DISP(x2,y2,Attr,'',1);
           end;
end;
{ ------------------------------------------------------------------------- }


procedure WEITER;

begin
   DISP(55,24,Inv ,'Eine Taste drcken....',0);
   repeat
   until keypressed
end;
{ ------------------------------------------------------------------------- }


function WordToHex (w : Word) : str4;      { converts a Word to 4 Hex Chars }

const HexArray : array[0..15] of char = '0123456789ABCDEF';

begin
   WordToHex := HexArray[w shr 12 and $F] + HexArray[w shr 8 and $F]
              + HexArray[w  shr 4 and $F] + HexArray[w and $F]
end;
{ ------------------------------------------------------------------------- }




function ZahlToStr(B: longint;L: integer): String;
{ Convert any integer type to a string
  B : Zahl, die in einen string umzuwandeln ist
  L : Lnge des strings                          }
var
  S1 : str1;
  S2 : str2;
  S3 : string[3];
  S4 : str4;
  S5 : string[5];
  S6 : string[6];
  S7 : string[7];
  S8 : str8;
  S9 : string[9];
  S10: string[10];
  S11: string[11];
  S12: string[12];
  S13: string[13];
  S14: string[14];
  S15: string[15];
  S16: string[16];
  S17: string[17];
begin
   case L of
      1  :  begin Str(B, S1);
                  while length(S1)<L do S1 := ' ' + S1; ZahlToStr := S1; end;
      2  :  begin Str(B, S2);
                  while length(S2)<L do S2 := ' ' + S2; ZahlToStr := S2; end;
      3  :  begin Str(B, S3);
                  while length(S3)<L do S3 := ' ' + S3; ZahlToStr := S3; end;
      4  :  begin Str(B, S4);
                  while length(S4)<L do S4 := ' ' + S4; ZahlToStr := S4; end;
      5  :  begin Str(B, S5);
                  while length(S5)<L do S5 := ' ' + S5; ZahlToStr := S5; end;
      6  :  begin Str(B, S6);
                  while length(S6)<L do S6 := ' ' + S6; ZahlToStr := S6; end;
      7  :  begin Str(B, S7);
                  while length(S7)<L do S7 := ' ' + S7; ZahlToStr := S7; end;
      8  :  begin Str(B, S8);
                  while length(S8)<L do S8 := ' ' + S8; ZahlToStr := S8; end;
      9  :  begin Str(B, S9);
                  while length(S9)<L do S9 := ' ' + S9; ZahlToStr := S9; end;
      10 :  begin Str(B, S10);
                  while length(S10)<L do S10 := ' ' + S10; ZahlToStr := S10; end;
      11 :  begin Str(B, S11);
                  while length(S11)<L do S11 := ' ' + S11; ZahlToStr := S11; end;
      12 :  begin Str(B, S12);
                  while length(S12)<L do S12 := ' ' + S12; ZahlToStr := S12; end;
      13 :  begin Str(B, S13);
                  while length(S13)<L do S13 := ' ' + S13; ZahlToStr := S13; end;
      14 :  begin Str(B, S14);
                  while length(S14)<L do S14 := ' ' + S14; ZahlToStr := S14; end;
      15 :  begin Str(B, S15);
                  while length(S15)<L do S15 := ' ' + S15; ZahlToStr := S15; end;
      16 :  begin Str(B, S16);
                  while length(S16)<L do S16 := ' ' + S16; ZahlToStr := S16; end;
      17 :  begin Str(B, S17);
                  while length(S17)<L do S17 := ' ' + S17; ZahlToStr := S17; end;
   end;
end;
{ ------------------------------------------------------------------------- }
{ ------------------------------------------------------------------------- }



{ ------------------------------------------------------------------------- }



begin
end.
{ -- Ende der UNIT SUPPORT.PAS -------------------------------------------- }